@@ -5,9 +5,9 @@ import os
 import random
 import re
 from typing import Any
+from typing import NoReturn
 from typing import overload
 from typing import Sequence
-from typing import TYPE_CHECKING
 
 import pre_commit.constants as C
 from pre_commit import parse_shebang
@@ -16,9 +16,6 @@ from pre_commit.prefix import Prefix
 from pre_commit.util import cmd_output_b
 from pre_commit.xargs import xargs
 
-if TYPE_CHECKING:
-    from typing import NoReturn
-
 FIXED_RANDOM_SEED = 1542676187
 
 SHIMS_RE = re.compile(r'[/\\]shims[/\\]')
